/*

    Meta-Environment - An environment for language prototyping.
    Copyright (C) 2000  Stichting Mathematisch Centrum, Amsterdam, The Netherlands. 
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

    $Id: StructureEdit.java,v 1.15 2000/04/17 15:27:27 markvdb Exp $
*/
package nl.cwi.gipe.stred;
import toolbus.aterm.*;

public class StructureEdit {
  // This is the interface class to the ToolBus. No constructors.

  public static Editor insertChars(Editor e, int i, String s) throws ParseError, SubtreeException {
    e = StructureEdit.makeTreeInvalid(e);
    //e = FocusMove.toTop(e);

    if (e.focuses().empty()) {
      return Updating.updateEdit(e.tree(), new FocusList(Focuses.createFocus(e.tree(),i)),i,s.length());
    } else {
      return Updating.updateEdit(e.tree(), e.focuses(), i, s.length());
    }
  }

  public static Editor deleteChars(Editor e, int i) {
    e = StructureEdit.makeTreeInvalid(e);
    //e = FocusMove.toTop(e);

    if (e.focuses().empty()) {
      return Updating.updateEdit(e.tree(), new FocusList(Focuses.createFocus(e.tree(),i)),i);
    } else {
      return Updating.updateEdit(e.tree(), e.focuses(), i);
    }
  }

    public static Editor replaceFocus(Editor e, ATerm t) {
    if (e.focuses().empty()) {
      return e;
    } else {
      // Currently, incoming terms are not annotated. We do that here. Eventually, we would like the parser to return annotated trees....
      t = PositionInformation.annotate(t);
      //debug
      //      System.err.println("in replaceFocus. The annotated term = "+t.toString());
      AFun w_left = new AFun(ATerm.the_world,"w-left");
      AFun w_right = new AFun(ATerm.the_world,"w-right");
      AFun term = new AFun(ATerm.the_world,"term");
      ATerm t2 = e.tree();
      FocusList fs = e.focuses();
      Focus f = fs.head();
      fs.tail();
      Path p = f.getPath();
      if (!p.isLeafPath()) {
	  Updating.wleft = (ATermAppl) AsFix.acc((ATermAppl) t,w_left);
	  Updating.wright =(ATermAppl) AsFix.acc((ATermAppl) t,w_right);
	  Updating.insertedWhitespaceLeft = false;
	  Updating.insertedWhitespaceRight = false;
	  t = AsFix.acc((ATermAppl) t, term);
      }
      ATerm t1 = Updating.updTerm(t2, f.getPath(), t);
      /* This is somewhat of a hack: We need to update the annotations, but
	 this has not been properly incorporated. This is a workaround, until
	 we can use the new generic traversal functions. 
      */
      t1 = PositionInformation.annotate(t1);
      /* End of hack */
      f.setSort(AsFixUtilities.typeOf(t));
      Area a = new Area(new Start(f.getArea().start()), new 
      Length(Area.areaLength(t)));
      f.setArea(a);
      fs.insertHead(f);
      return new Editor(t1, fs);
    }
  }

  public static Editor newEditor(String s) {
      return newEditor(new Length(s.length()));
  }

    public static Editor newEditor(Length length) {
	//    System.err.println("The string is: "+s+", and has length: "+Integer.toString(s.length()));
	ATermAppl ws = new ATermAppl(ATerm.the_world,"w",new ATerms(ATerm.the_world,new ATermAppl(ATerm.the_world,"",true)));
	ws.setAnnotation(Length.sym,length);
	ATermList t = new ATermList(ATerm.the_world,new ATerms(ATerm.the_world,ws));
	Path p = new Path();
	Sort sort = new Sort("invalid");
	Start st = new Start(1);
	Area a = new Area(st,length);
	Focus f = new Focus(p,sort,a);
	f.setDirty();
	return new Editor(t,new FocusList(f));
    }

  public static Editor replaceWithMeta(Editor e) throws 
  Exception,ParseError {
    if (e.focuses().empty() == false){
      Sort s = e.focuses().head().getSort();
      ATerm t2 = new MetaVar(s);
      if (t2 != null) {
	return replaceFocus(e,t2);
      }
    }
    return e;
  }
  
  public static Editor moveFocus(Editor e, Move m) throws MoveException {
    if (e.focuses().empty() == false) {
      Editor e2 = FocusMove.move(e,m);
      Sort s = e2.focuses().head().getSort();
      if (s.equals(Sort.error) == false) {
	return e2;
      }
    }
    return e;
  }

  public static ATerm getFocus(Editor e, int i) throws ParseError {
    return Focuses.createFocus(e.tree(), i).toATerm();
  }

  public static void main(String argv[]) throws Exception {
    /*
    Editor e = newEditor("");
    System.out.println(e.tree().toString());
    Editor e2 = insertChars(e,0,"j");
    
    System.out.println(e2.tree().toString());
    System.out.println(e2.focuses().toString());
    e2 = insertChars(e2,1,"j");
    System.out.println(e2.tree().toString());
    System.out.println(e2.focuses().toString());
    */
    /* ATerm t = ATermParser.makeSimple("w(\"     \")");
    t = PositionInformation.annotate(t);
    System.err.println("t after annotation: "+t.toString());
    ATermAppl at = (ATermAppl) t;
    at.setArgs(null);
    System.err.println("After setArgs: "+at.toString());
    */
  }

    public static Editor makeTreeInvalid(Editor editor) {
	editor = FocusMove.toTop(editor);
	Focus topFocus = editor.focuses().head();
	Area area = topFocus.getArea();
	
	return newEditor(area.getLength());
    }
}
    
      
      
